local config_name = "MDI TargetList"
declare("targetlist", {
	pc_only = gkini.ReadInt(config_name, "PC Only", 0) == 1,
	fontsize = gkini.ReadInt(config_name, "FontSize", Font.H5),
	playerlist = {},
	shiplist = {},
	id = "targetlist",
	refresh_timer = Timer(),
	refresh_delay = 1000,
	current_target_listindex,
	current_target_nodeid,
	current_target_objectid,
	default_target_color = "65 255 65 65 *"
	-- health_width, -- set below
	-- name_width, -- set below
	-- distance_width, -- set below
})
targetlist.current_target_color = gkini.ReadString(config_name, "Highlight", targetlist.default_target_color)

local function IsDeclared(value) return pcall(loadstring("return "..value)) end

local function update_display()
	targetlist.current_target_listindex = nil
	for i = 1, 10 do
		local health, shield, name, distance, faction = "", "", "", "", 0

		if (targetlist.shiplist[i]) then
			local charid = targetlist.shiplist[i].charid
			local nodeid = GetPlayerNodeID(charid); targetlist.shiplist[i].nodeid = nodeid
			local objectid = GetPrimaryShipIDOfPlayer(charid); targetlist.shiplist[i].objectid = objectid
			local guild = GetGuildTag(charid)
			name = targetlist.shiplist[i].name
			distance = targetlist.shiplist[i].distance
			health, shield = GetPlayerHealth(charid)
			health = math.ceil(health or 0)
			faction = GetPlayerFaction(charid)
			if (guild and guild ~= "") then name = "["..tostring(guild).."] "..name end
			
			local row = targetlist.display[i+1][2]
			row[2].title = health.."%"
			row[2].fgcolor = calc_health_color(health * 0.01)
			row[3][1][2].visible = shield and "YES" or "NO"
			row[3][2].value = name
			row[3][2].caret = 1
			row[3][2].fgcolor = FactionColor_RGB[faction]
			row[4].title = distance
			targetlist.display[i+1].visible = "YES"
			
			if (shield) then
				shield = math.ceil(shield)
				row[3][1][2].value = shield
			end

			if (nodeid == targetlist.current_target_nodeid) then
				targetlist.display[i+1][1].bgcolor = targetlist.current_target_color
				targetlist.current_target_listindex = i
			else
				targetlist.display[i+1][1].bgcolor = "0 0 0 0 *"
			end
				
		else
			targetlist.display[i+1].visible = "NO"
		end
	end
end

local function scan_player(charid)
	if (not charid) then return end
	-- local distance = GetPlayerDistance(charid)
	local distance = GetRadarDistance(charid)
	local name = GetPlayerName(charid)
--	test.varprint(' / ', 'id', charid, 'name', name, 'pd', GetPlayerDistance(charid), 'rd', GetRadarDistance(charid))
	if (distance) then
		local name = GetPlayerName(charid)
		if (not targetlist.pc_only or name:match("^[^*]")) then

			table.insert(targetlist.shiplist, {
				charid = charid,
				name = name,
				distance = math.floor(distance),
			})
		end
	end
	
	return scan_player(table.remove(targetlist.playerlist))
end

local function scan_sector()
	targetlist.refresh_timer:Kill()
	targetlist.playerlist = {}
	targetlist.shiplist = {}

	ForEachPlayer(function(charid)
		if (charid ~= GetCharacterID()) then table.insert(targetlist.playerlist, charid) end
	end)
	scan_player(table.remove(targetlist.playerlist))
	table.sort(targetlist.shiplist, function(a, b) return a.distance < b.distance end)
	update_display()

	targetlist.refresh_timer:SetTimeout(targetlist.refresh_delay, scan_sector)
end

local function ship_display(num, health, name, distance)
	num = num or ""
	health = health or ""
	name = name or ""
	distance = distance or ""

	num = iup.label{title = num, size = targetlist.num_width, font = targetlist.fontsize, fgcolor = "35 135 255 255 *", expand = "NO"}
	health = iup.label{title = health, size = targetlist.health_width, font = targetlist.fontsize, expand = "NO"} 
	name = iup.text{value = name, size = targetlist.name_width, font = targetlist.fontsize, expand = "NO", active = "NO", border = "NO", bgcolor = "0 0 0 255"}
	distance = iup.label{title = distance, size = targetlist.distance_width, font = targetlist.fontsize, alignment = "ARIGHT", expand = "NO"}
	
	local shield = iup.stationprogressbar{
		visible = "NO", active = "NO", expand = "HORIZONTAL", title = "",
		size = "x"..math.ceil(targetlist.fontsize/4),
		minvalue = 0, maxvalue = 100, uppercolor = "88 88 88 255 *", lowercolor = "65 65 255 255 *",
		value = 0, uv = "0 0.25 1 0.75"
	}
	
	local display = iup.zbox{
		iup.label{title = "", expand = "YES", bgcolor = "0 0 0 0 *"},
		iup.hbox{num, health, 
		iup.zbox{
			iup.vbox{iup.fill{size = math.ceil(targetlist.fontsize*3/5)}, shield},
			name, 
			all = "YES",
		}, distance, gap = 5, size = "200x",},
		alignment = "CENTER",
		all = "YES",
	}

	if (not targetlist.health_width) then
		num:map();      targetlist.num_width = tonumber(num.size:match("(%d+)x"))
		health:map();   targetlist.health_width = tonumber(health.size:match("(%d+)x"))
		name:map();     targetlist.name_width = tonumber(name.size:match("(%d+)x"))
		distance:map(); targetlist.distance_width = tonumber(distance.size:match("(%d+)x"))
		num.title = ""
		name.value = name.value:gsub("W", "")
		name.size = targetlist.name_width
	end

	--shield.size = string.format("%dx%d", targetlist.name_width, 3)--math.ceil(targetlist.fontsize/6))
	
	return display
end

targetlist.display = iup.vbox{
	ship_display(99, "Health", "Name"..string.rep("W", 10), "Distance"),
	ship_display(1),
	ship_display(2),
	ship_display(3),
	ship_display(4),
	ship_display(5),
	ship_display(6),
	ship_display(7),
	ship_display(8),
	ship_display(9),
	ship_display(10),
}
targetlist.display_container = iup.hudrightframe{targetlist.display, cx = targetlist.x, cy = targetlist.y, expand = "NO"}

local function change_target(i)
	if (targetlist.shiplist[i]) then
		radar.SetRadarSelection(targetlist.shiplist[i].nodeid, targetlist.shiplist[i].objectid)
	end
end
for i = 1, 10 do
	RegisterUserCommand("target"..i, function() change_target(i) end)
end

RegisterUserCommand("targetnext", function()
	local i = targetlist.current_target_listindex or 0
	i = (i + 1) % 10
	if (i == 0) then i = 10 end
	if (i > #targetlist.shiplist) then i = 1 end
	change_target(i)
end)

RegisterUserCommand("targetprev", function()
	local i = targetlist.current_target_listindex or 0
	i = (i - 1) % 10
	if (i == 0) then i = 10 end
	if (i > #targetlist.shiplist) then i = #targetlist.shiplist end
	change_target(i)
end)

local function togglepconly() 
	targetlist.pc_only = not targetlist.pc_only
	gkini.WriteInt(config_name, "PC Only", targetlist.pc_only and 1 or 0)
	scan_sector()
end
RegisterUserCommand("togglepconly", togglepconly)
	
RegisterUserCommand("settargetbinds", function()
	for i = 1, 10 do 
		gkinterface.GKProcessCommand("bind "..(i%10).." target"..i)
	end
	gkinterface.GKProcessCommand("bind - togglepconly")
end)

RegisterUserCommand("targetlist", function(_, data)
	if (not data) then return end
	
	local cmd = string.lower(table.remove(data, 1))

	if (cmd == "pos" and not targetlist.hasmdi) then 

		local x, y = tonumber(data[1]), tonumber(data[2])
		local parent = iup.GetParent(targetlist.display)
		if (x and y) then
			parent.cx, parent.cy = x, y
			HUD.cboxlayer.REFRESH = "YES"
			gkini.WriteInt(config_name, "x", x)
			gkini.WriteInt(config_name, "y", y)
		else
			print("\127FFFFFFTargetlist position: "..parent.cx.."x"..parent.cy)
		end
		
	elseif (cmd == "font") then

		local newsize = tonumber(data[1])
		if (newsize) then
			targetlist.fontsize = newsize
			gkini.WriteInt(config_name, "FontSize", newsize)
			print("\127FFFFFFFont size has been changed to "..newsize..".")
			print("\127FFFFFFYou must reload the interface for the change to take effect.")
			
		else
			print("\127FFFFFFThe current font size is "..targetlist.fontsize..".")
		end
		
	elseif (cmd == "highlight") then
		
		local r, g, b, a = tonumber(data[1]), tonumber(data[2]), tonumber(data[3]), tonumber(data[4])
		if (r and g and b) then
			a = a or 65
			targetlist.current_target_color = string.format("%d %d %d %d *", r, g, b, a)
			gkini.WriteString(config_name, "Highlight", targetlist.current_target_color)
		
		else
			if (data[1] == "reset") then
				targetlist.current_target_color = targetlist.default_target_color
				gkini.WriteString(config_name, "Highlight", targetlist.current_target_color)
			else
				print("\127FFFFFFThe current highlight color is "..targetlist.current_target_color:match("(.*) %*"))
			end
		end
	end
end)

RegisterEvent(function()
	scan_sector()
end, "HUD_SHOW")

RegisterEvent(function()
	targetlist.refresh_timer:Kill()
end, "HUD_HIDE")

local function target_changed()
	if (targetlist.current_target_listindex) then targetlist.display[targetlist.current_target_listindex + 1][1].bgcolor = "0 0 0 0 *" end
	targetlist.current_target_nodeid, targetlist.current_target_objectid = radar.GetRadarSelectionID()
	targetlist.current_target_listindex = nil
	for i, v in ipairs(targetlist.shiplist) do
		if (i > 10) then break end
		if (v.nodeid == targetlist.current_target_nodeid) then
			targetlist.display[i+1][1].bgcolor = targetlist.current_target_color
			targetlist.current_target_listindex = i
			break
		end
	end
end
RegisterEvent(target_changed, "TARGET_CHANGED")

RegisterEvent(function()
	if (not targetlist.hasmdi) then
		iup.Detach(targetlist.display_container)
	end
end, "PLAYER_LOGGED_OUT")

RegisterEvent(function()
	if (not targetlist.hasmdi) then
		iup.Append(iup.GetNextChild(HUD.cboxlayer), targetlist.display_container)
		HUD.cboxlayer:map()
	end
end, "PLAYER_ENTERED_GAME")

RegisterEvent(function()
	targetlist.hasmdi = IsDeclared("mdi")
	--do return end
	if (targetlist.hasmdi) then
		local contextmenu = {{
			title = function()
				if (targetlist.pc_only) then return "Show NPCs" end
				return "Hide NPCs"
			end,
			action = togglepconly,
		}}
		mdi:CreateWindow(targetlist.id, targetlist.display, {title = "Target List", cx = 350, cy = 350}, contextmenu)
	else
		targetlist.x = gkini.ReadInt("MDI TargetList", "x", 350)
		targetlist.y = gkini.ReadInt("MDI TargetList", "y", 350)
		targetlist.display_container.cx = targetlist.x
		targetlist.display_container.cy = targetlist.y
	end
	target_changed()
end, "PLUGINS_LOADED")
